package exnihilocreatio.client.renderers;

import exnihilocreatio.blocks.BlockCrucibleBase;
import exnihilocreatio.client.models.ModelVertex;
import exnihilocreatio.texturing.Color;
import exnihilocreatio.texturing.SpriteColor;
import exnihilocreatio.tiles.TileCrucibleBase;
import net.minecraft.block.Block;
import net.minecraft.client.renderer.BufferBuilder;
import net.minecraft.client.renderer.texture.TextureAtlasSprite;
import net.minecraft.client.renderer.vertex.VertexFormatElement;
import net.minecraft.util.EnumFacing;
import net.minecraft.util.math.BlockPos;
import net.minecraftforge.client.model.animation.FastTESR;

public class RenderCrucible extends FastTESR<TileCrucibleBase> {
    private static ModelVertex[] model = new ModelVertex[4];

    static {
        model[0] = new ModelVertex(EnumFacing.UP, 0.125, 0.6875, 0.125, 0, 0);
        model[1] = new ModelVertex(EnumFacing.UP, 0.875, 0.6875, 0.125, 1, 0);
        model[2] = new ModelVertex(EnumFacing.UP, 0.875, 0.6875, 0.875, 1, 1);
        model[3] = new ModelVertex(EnumFacing.UP, 0.125, 0.6875, 0.875, 0, 1);
    }

    @Override
    public void renderTileEntityFast(TileCrucibleBase te, double x, double y, double z, float partialTicks, int destroyStage, float partial, BufferBuilder buffer) {
        if (te == null) return;
        final BlockPos pos = te.func_174877_v();
        final Block block = func_178459_a().func_180495_p(pos).func_177230_c();

        // Check because sometimes the renderer gets called while the block is breaking/decaying
        if (!(block instanceof BlockCrucibleBase)) return;

        // Draw
        SpriteColor[] sprites = te.getSpriteAndColor();

        buffer.func_178969_c(x, y, z);
        addSpriteColor(te, sprites[0], buffer, te.getSolidProportion());
        addSpriteColor(te, sprites[1], buffer, te.getFluidProportion());
        buffer.func_178969_c(0, 0, 0);
    }
    private void addSpriteColor(TileCrucibleBase te, SpriteColor sprite, BufferBuilder buffer, float fill) {
        if (sprite == null) return;

        final BlockPos pos = te.func_174877_v();
        // Light levels
        final int mixedBrightness = te.func_145831_w().func_180495_p(pos).func_185889_a(te.func_145831_w(), te.func_174877_v());
        final int skyLight = mixedBrightness >> 16 & 0xFFFF;
        final int blockLight = mixedBrightness & 0xFFFF;
        // Texturing
        TextureAtlasSprite icon = sprite.getSprite();
        Color color = sprite.getColor();

        // Draw
        for (final ModelVertex vert : model) {
            for (final VertexFormatElement e : buffer.func_178973_g().func_177343_g()) {
                switch (e.func_177375_c()) {
                    case COLOR:
                        buffer.func_181666_a(color.r, color.g, color.b, color.a);
                        break;

                    case NORMAL:
                        buffer.func_181663_c(vert.face.func_82601_c(), vert.face.func_96559_d(), vert.face.func_82599_e());
                        break;

                    case POSITION:
                        final double vertX = vert.x;
                        final double vertY = vert.y * fill + 0.25;
                        final double vertZ = vert.z;

                        buffer.func_181662_b(vertX, vertY, vertZ);
                        break;

                    case UV:
                        if (e.func_177369_e() == 1) {
                            buffer.func_187314_a(skyLight, blockLight);
                        } else {
                            buffer.func_187315_a(icon.func_94214_a(vert.u), icon.func_94207_b(16.0 - vert.v));
                        }
                        break;

                    default:
                        break;
                }
            }
            buffer.func_181675_d();
        }
    }

}
