package exnihilocreatio.util;

import lombok.AllArgsConstructor;
import lombok.Getter;
import lombok.Setter;
import net.minecraft.block.Block;
import net.minecraft.block.state.IBlockState;
import net.minecraft.init.Blocks;
import net.minecraft.item.Item;
import net.minecraft.item.ItemStack;
import net.minecraft.nbt.NBTTagCompound;
import net.minecraft.util.ResourceLocation;

@AllArgsConstructor
public class ItemInfo {

    @Getter
    private Item item;
    @Getter
    @Setter
    private int meta;

    public ItemInfo(ItemStack stack) {
        item = stack == null ? null : stack.func_77973_b();
        meta = stack == null ? -1 : stack.func_77960_j();
    }

    public ItemInfo(Block block, int blockMeta) {
        item = Item.func_150898_a(block);
        meta = block == Blocks.field_150350_a ? -1 : blockMeta;
    }

    public ItemInfo(String string) {
        String[] split = string.split(":");

        switch (split.length) {
            case 1:
                item = Item.func_111206_d("minecraft:" + split[0]);
                break;
            case 2:
                try {
                    meta = split[1].equals("*") ? -1 : Integer.parseInt(split[1]);
                    item = Item.func_111206_d("minecraft:" + split[0]);
                } catch (NumberFormatException e) {
                    meta = -1;
                    item = Item.func_111206_d(split[0] + ":" + split[1]);
                }
                break;
            case 3:
                try {
                    meta = split[2].equals("*") ? -1 : Integer.parseInt(split[2]);
                    item = Item.func_111206_d(split[0] + ":" + split[1]);
                } catch (NumberFormatException e) {
                    meta = -1;
                }
                break;
            default:
                meta = -1;
                break;
        }
    }

    public ItemInfo(IBlockState state) {
        item = state == null ? null : Item.func_150898_a(state.func_177230_c());
        meta = state == null ? -1 : state.func_177230_c().func_176201_c(state);
    }

    public static ItemInfo getItemInfoFromStack(ItemStack stack) {
        return new ItemInfo(stack);
    }

    public static ItemInfo readFromNBT(NBTTagCompound tag) {
        Item item_ = Item.field_150901_e.func_82594_a(new ResourceLocation(tag.func_74779_i("item")));
        int meta_ = tag.func_74762_e("meta");

        return new ItemInfo(item_, meta_);
    }

    public String toString() {
        return Item.field_150901_e.func_177774_c(item) + (meta == -1 ? "" : (":" + meta));
    }

    public ItemStack getItemStack() {
        return item == null ? null : new ItemStack(item, 1, meta == -1 ? 0 : meta);
    }

    public NBTTagCompound writeToNBT(NBTTagCompound tag) {
        tag.func_74778_a("item", Item.field_150901_e.func_177774_c(item) == null ? "" : Item.field_150901_e.func_177774_c(item).toString());
        tag.func_74768_a("meta", meta);

        return tag;
    }

    public boolean isValid() {
        return meta <= Short.MAX_VALUE && item != null;
    }

    public int hashCode() {
        return item == null ? 41 : item.hashCode();
    }

    public boolean equals(Object other) {
        if (other instanceof ItemInfo) {
            ItemInfo info = (ItemInfo) other;

            if (item == null || info.item == null) {
                return false;
            }

            if (meta == -1 || info.meta == -1) {
                return item.equals(info.item);
            } else {
                return meta == info.meta && item.equals(info.item);
            }
        }

        return false;
    }
}
